__title__ = 'velocity'
__description__ = 'DNS caching for humans'
__url__ = 'https://github.com/s0md3v/velocity'
__version__ = '0.0.1'
__author__ = 'Somdev Sangwan'
__author_email__ = 's0md3v@gmail.com'
__license__ = 'GPL v3'
__copyright__ = 'Copyright 2020 Somdev Sangwan'



#!/usr/bin/env python
import os
import re
import sys

from codecs import open

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist')
    os.system('twine upload dist/*')
    sys.exit()

packages = ['velocity']

about = {}
with open(os.path.join(here, 'velocity', '__version__.py'), 'r', 'utf-8') as f:
    exec(f.read(), about)

with open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=packages,
    package_data={'': ['LICENSE'],},
    package_dir={'velocity': 'velocity'},
    include_package_data=True,
    python_requires=">=2.7",
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3'
    ],
    project_urls={
        'Source': 'https://github.com/s0md3v/velocity',
    },
)



import sys
import json
import socket
import _socket

dns_cache = {}  # contains {hostname : ip} entries
addr_cache = {}  # contains socket.getaddrinfo's cached responses

cwd = sys.path[0]
dns_path = cwd + '/dns_cache.json'
addr_path = cwd + '/addr_cache.json'

def read_file(path):
    """Reads a given file, returns str(file_content)"""
    with open(path, 'a+') as f:
        f.seek(0, 0)
        return '\n'.join(f.readlines())


def flush_db():
    """Flushes local cache"""
    for path in [addr_path, dns_path]:
        with open(path, 'w+') as f:
            f.write('{}\n')


def save_db():
    """Saves in-memory cache locally"""
    for path, db in zip(
        [addr_path, dns_path],
            [addr_cache, dns_cache]):
        with open(path, 'w') as f:
            f.write(json.dumps(db))


def load_db():
    """Load locally saved cache"""
    try:
        addr_cache = json.loads(read_file(addr_path))
        dns_cache = json.loads(read_file(dns_path))
    except json.decoder.JSONDecodeError:
        pass


og_addr = socket.getaddrinfo
og_connect = _socket.socket.connect

def resolve(hostname, port=False):
    """
    Resolves hostname to IP address.
    IPv6 addresses get mapped to IPv6 if a port isn't supplied
    returns str(IP_address)
    """
    if port:
        return _socket.getaddrinfo(hostname, port)[0][4][0]
    else:
        return _socket.gethostbyname(hostname)

def cache(hostname, port=False):
    """Resolve and add hostname to cache"""
    dns_cache[hostname] = resolve(hostname, port) 

def patched_connect(sock, address):
    """
    Intercepting function for _socket.socket.connect
    returns _socket.socket.connect's result
    """
    hostname, port = address[:2]
    if hostname not in dns_cache:
        dns_cache[hostname] = resolve(hostname, port)
    hostname = dns_cache[hostname]
    return og_connect(sock, (hostname, port))

def patched_addr(host, *args, **kwargs):
    """
    Intercepting function for socket.getaddrinfo
    returns socket.getaddrinfo's result or
    its cached equivalent when available
    """
    if host in addr_cache:
        return addr_cache[host]
    addrlist = og_addr(host, *args, **kwargs)
    addr_cache[host] = addrlist
    return addrlist

socket.getaddrinfo = patched_addr
socket.socket.connect = patched_connect
